//+------------------------------------------------------------------+
//|                                                       Simple.mq4 |
//+------------------------------------------------------------------+

// Trend Detection function
#define BULL 1
#define BEAR 2
// Input variables
//-------------------------------------------------------------------+
extern bool   MoneyM         = true;
extern double Lots           = 0.01;
extern double MaximumRisk    = 0.3;
//-------------------------------------------------------------------+
extern int    StopLoss       = 0;
extern int    TakeProfit     = 0;
//-------------------------------------------------------------------+
extern bool   Trailing       = false;
extern int    MinProfit      = 610;
extern int    TrailingStop   = 233;
extern int    TrailingStep   = 233;
//-------------------------------------------------------------------+
extern bool   Bezubitok      = false;
extern int    MinProfitB     = 144;
extern int    BezLevel       = 13;
//-------------------------------------------------------------------+
extern string IndiName       = "BBMACD"; //Indicator's Name
extern int    BuyBuffer      = 0;
extern int    SellBuffer     = 1;
extern int    IndicatorTF    = 60;
extern int    Shift          = 1;
//-------------------------------------------------------------------+
extern bool   UseSound       = true;
//-------------------------------------------------------------------+
extern int    Slippage       = 89;
extern int    Magic          = 585298229;
extern string SuccesSound    = "ok.wav";
extern string Order_Comment  = "Simple";
//-------------------------------------------------------------------+
 
// Global variables
int Total_Open_Orders = 1; // we only want to open one order at a time and only manage the one order
int cnt = 0;               // counter variable, used in for() loops
bool init_variables;       // init variable when program starts
datetime PreviousBar;      // record the candle/bar time
int LastTrendDirection;    // record the previous trends direction
int FileHandle;            // variable for file handling 
 
int init()
  {
   if(IsTesting()==false)
   {
      init_variables = true;  // Allows us to init variables in the start function because 
                           // we cannot do this in the init function because the variables 
                           // use values from the chart 
                           
      //create data file if it does not exist and load variables if file does exist
      FileHandle=FileOpen("simple.dat",FILE_BIN | FILE_READ);
      if(FileHandle<1)
      {
         Print("simple.dat not found.");
         FileHandle=FileOpen("simple.dat",FILE_BIN | FILE_WRITE);
         if(FileHandle>0)
         {
            LastTrendDirection=0;
            FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
            FileClose(FileHandle);
            Print("simple.dat has been successfully created.");
         }
         else
         {
            FileClose(FileHandle);
            Print("Failed to create simple.dat file.");
         }  
      }
      else
      {
          LastTrendDirection=FileReadInteger(FileHandle,SHORT_VALUE);
          Print("Variables loaded from file.");
          FileClose(FileHandle);
      }
   }      
   return(0);
  }
int deinit()
  {
   return(0);
  }
 
int start()
  {
   if(Bezubitok) PlaseBezubitok();
   if(Trailing) TrailPositions();    
   // moving average only run expert advisor if there is enough candle/bars in history
   if(Bars < 3)
   {
      if(IsTesting()==false)
      {
         Comment("Long moving average does not have enough Bars in history to open a trade!\n", 
               "Must be at least 610 bars to perform technical analysis.");
      }
      else
      {
         Comment("Back testing currently Long moving average does not have enough Bars to open a trade.\n", 
               "Must be at least 610 bars to perform technical analysis.\n",
               "Please be patient and wait...\n",
               "Bars in history ",Bars);
      }
      return(0);
   }
   if(MoneyM == true)
   Lots = NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,2);
   else Lots = Lots;    
   // init variables when the expert advisor first starts running
   if(init_variables == true)
   {
      PreviousBar = Time[0];     // record the current canle/bar open time
      
      // place code here that you only wnat to run one time
       
      init_variables = false;    // change to false so we only init variable once
   }
   
   // record trends direction
   if(LastTrendDirection==0)
      {
         if(TrendDetection()==BULL)
         {
            LastTrendDirection=BULL;
         }
         if(TrendDetection()==BEAR)
         {
            LastTrendDirection=BEAR;
         }
//-----
      }
      
   // perform analysis and open orders on new candle/bar 
   if(NewBar() == true)
   {  
      // only perform analysis and close order if we only have one order open
      if(TotalOpenOrders() == Total_Open_Orders && SelectTheOrder() == true)
      {     
         if(OrderType() == OP_BUY && TrendDetection() == BEAR)
         {
            OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,White);
         }
         if(OrderType() == OP_SELL && TrendDetection() == BULL)
         {
            OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,White);
         }                
      }
      // only perform analysis and open new order if we have not reached our Total_Open_Orders max
      if(TotalOpenOrders() < Total_Open_Orders)
      {
         // open buy
         if(TrendDetection() == BULL)
         {
            if(StopLoss>0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,Ask-(StopLoss*Point),Ask+(TakeProfit*Point),Order_Comment,Magic,0,Green);
            }
            if(StopLoss>0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,Ask-(StopLoss*Point),0,Order_Comment,Magic,0,Green);
            }
            if(StopLoss==0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,Ask+(TakeProfit*Point),Order_Comment,Magic,0,Green);
            }
            if(StopLoss==0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,Order_Comment,Magic,0,Green);
            }          
            LastTrendDirection=BULL;
            // save variables to file
            SaveVariables();
         }         
         // open sell
         if(TrendDetection() == BEAR)
         {
            if(StopLoss>0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,Bid+(StopLoss*Point),Bid-(TakeProfit*Point),Order_Comment,Magic,0,Red);
            }
            if(StopLoss>0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,Bid+(StopLoss*Point),0,Order_Comment,Magic,0,Red);
            }
            if(StopLoss==0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,Bid-(TakeProfit*Point),Order_Comment,Magic,0,Red);
            }
            if(StopLoss==0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,0,Order_Comment,Magic,0,Red);
            }           
            LastTrendDirection=BEAR;
            // save variables to file
            SaveVariables();
         }
      }      
      // when back testing only display chart info every candle/bar so we do not slow down back tests
      if(IsTesting() == true)
      {
         Display_Info();
      }
   }
   // when not back testing display chart info every tick
   if(IsTesting() == false)
   {
      Display_Info();
   }   
   return(0);
  }
/////////////////////////////////////////////////////////////////////////
// Common functions                                                   //
///////////////////////////////////////////////////////////////////////
int SaveVariables()
{
   if(IsTesting()==false)
   {
      //save variables to file
      FileHandle=FileOpen("simple.dat",FILE_BIN | FILE_WRITE);
      if(FileHandle<1)
      {
         Print("simple.dat not found.");
         FileHandle=FileOpen("simple.dat",FILE_BIN | FILE_WRITE);
         if(FileHandle>0)
         {
            FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
            FileClose(FileHandle);
            Print("simple.dat has been successfully created.");
         }  
         else
         {
            FileClose(FileHandle);
            Print("Failed to create simple.dat file.");
         }
      }
      else
      {
         FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
         FileClose(FileHandle);
         Print("Simple variables successfully saved to file.");
      }  
   }
return(0);
}
// This function returns the total amount of orders the expert advisor has open  
int TotalOpenOrders()
{
   cnt=OrdersTotal();
   int TotalOpenOrders = 0;
   
   if(cnt==0)
   {
      return(0);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            TotalOpenOrders++;
         }
      }
   }
   return(TotalOpenOrders);
}
// This function finds the open order and selects it 
int SelectTheOrder()
{
   cnt=OrdersTotal();
      
   if(cnt==0)
   {
      return(false);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            return(true);
         }
      }
   }
   return(false);
}
// This function return the value true if the current bar/candle was just formed
bool NewBar()
{
   if(PreviousBar<Time[0])
   {
      PreviousBar = Time[0];
      return(true);
   }
   else
   {
      return(false);
   }
   return(false);    // in case if - else statement is not executed
}
// is trend up/bullish or is it down/bearish
//--------------------------------------// Trend Detection //-----------------------------------//
int TrendDetection()
{           
//--------------------------------------------------------------------+
      double GT1now = iCustom(NULL,IndicatorTF,IndiName,BuyBuffer,Shift);
      double GT1pre = iCustom(NULL,IndicatorTF,IndiName,BuyBuffer,Shift+1);
      double GT2now = iCustom(NULL,IndicatorTF,IndiName,SellBuffer,Shift);
      double GT2pre = iCustom(NULL,IndicatorTF,IndiName,SellBuffer,Shift+1);    
//--------------------------------------------------------------------+
   // BULL trend
   if(GT1now>=0 && GT1now!=EMPTY_VALUE && (GT1pre==EMPTY_VALUE || GT1pre<=0))
   {
      return(BULL);
   }
   // BEAR trend
   if(GT2now>=0 && GT2now!=EMPTY_VALUE && (GT2pre==EMPTY_VALUE || GT2pre<=0))
   {
      return(BEAR);
   }
   return(0);
}
 
void Display_Info()
{
   Comment("Lots:  ",Lots,"\n");
   return(0);
}
//+------------------------------------------------------------------+
//|-----------------------// Trail positions //----------------------+
//+------------------------------------------------------------------+
void TrailPositions()
{
  int Orders = OrdersTotal();
  for (int i=0; i<Orders; i++)
  {
    if (!(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))) continue;
    if (OrderSymbol() != Symbol()) continue;
    {    
      if (OrderType() == OP_BUY)  
      {
        if (Bid-OrderOpenPrice() > MinProfit*Point) 
        {
          if (OrderStopLoss() < Bid-(TrailingStop+TrailingStep-1)*Point) 
          {
          OrderModify(OrderTicket(),OrderOpenPrice(),Bid-TrailingStop*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }
      }   
      if (OrderType() == OP_SELL)  
      {
        if (OrderOpenPrice()-Ask > MinProfit*Point) 
        {
          if (OrderStopLoss() > Ask+(TrailingStop+TrailingStep-1)*Point) 
          {
          OrderModify(OrderTicket(),OrderOpenPrice(),Ask+TrailingStop*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }
      }   
    }   
  }  
}
//+------------------------------------------------------------------+
//|--------------------------// Bezubytok //-------------------------+
//+------------------------------------------------------------------+
void PlaseBezubitok()
{
  int Orders = OrdersTotal();
  for (int i=0; i<Orders; i++)
  {
    if (!(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))) continue;
    if (OrderSymbol() != Symbol()) continue;
    {    
      if (OrderType() == OP_BUY && OrderStopLoss() < OrderOpenPrice())  
      {
        if (Bid-OrderOpenPrice() > MinProfitB*Point) 
        {
          if (OrderStopLoss() < Bid-(BezLevel-1)*Point) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(),OrderOpenPrice()+BezLevel*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }
      }
      if (OrderType() == OP_BUY && OrderStopLoss() == 0)  
      {
        if (Bid-OrderOpenPrice() > MinProfitB*Point) 
        {
          if (OrderStopLoss() < Bid-(BezLevel-1)*Point) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(),OrderOpenPrice()+BezLevel*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }
      }         
      if (OrderType() == OP_SELL && OrderStopLoss() > OrderOpenPrice())  
      {
        if (OrderOpenPrice()-Ask > MinProfitB*Point) 
        {
          if (OrderStopLoss() > Ask+(BezLevel-1)*Point) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(),OrderOpenPrice()-BezLevel*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }
      }  
      if (OrderType() == OP_SELL && OrderStopLoss() == 0)  
      {
        if (OrderOpenPrice()-Ask > MinProfitB*Point) 
        {
          if (OrderStopLoss() > Ask+(BezLevel-1)*Point) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(),OrderOpenPrice()-BezLevel*Point,OrderTakeProfit(),0,Gold);
            if(UseSound == true)
            {
            PlaySound(SuccesSound);
            }           
          }
        }        
      }   
    }   
  }  
}
//+------------------------------------------------------------------+
//|---------------------------// END //------------------------------|
//+------------------------------------------------------------------+